/*
 *  Copyright (C) 2015  Thadeu Lima de Souza Cascardo <cascardo@cascardo.eti.br>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>

#include "declaracao.h"
#include "cmd.h"
#include "base.h"
#include "contribuinte.h"
#include "conjuge.h"
#include "rendimento.h"
#include "isento.h"
#include "pagamento.h"
#include "bem.h"
#include "dependente.h"
#include "calcula.h"
#include "gera.h"

static int realprocess(struct declaracao *dec, int fd)
{
	char *line = NULL;
	size_t lsz = 0;
	FILE *f;
	int r;
	int n = 1;
	f = fdopen(fd, "r");
	if (!f)
		return -errno;
	while ((r = getline(&line, &lsz, f)) > 0) {
		r = cmd_run(dec, line);
		if (r < 0) {
			fprintf(stderr, "Não foi possível executar comando na linha %d: %s\n",
				n, strerror(-r));
		}
		n++;
	}
	free(line);
	return r;
}

static int process(char *filename)
{
	int r = 0;
	int fd;
	struct declaracao *dec;
	dec = declaracao_new(-1);
	if (!dec)
		return -errno;
	fd = open(filename, O_RDONLY);
	if (fd < 0) {
		r = -errno;
		goto out_open;
	}
	realprocess(dec, fd);
	close(fd);
out_open:
	declaracao_free(dec);
	return r;
}

static void usage(void)
{
	fprintf(stderr, "declara <filename>\n");
	exit(1);
}

int main(int argc, char **argv)
{
	char *filename;
	int r;
	if (argc < 2)
		usage();

	cmd_init();
	base_cmd_init();
	contribuinte_cmd_init();
	conjuge_cmd_init();
	rendimento_cmd_init();
	isento_cmd_init();
	pagamento_cmd_init();
	bem_cmd_init();
	dependente_cmd_init();
	calcula_cmd_init();
	gera_cmd_init();
	sistema_cmd_init();

	filename = argv[1];
	r = process(filename);
	if (r)
		return 1;
	return 0;
}
